@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 7/13/2019.
-//  Copyright (c) 2019-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2019-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -22,6 +22,7 @@
 #include "QtSLiMWindow.h"
 #include "QtSLiMFindRecipe.h"
 #include "QtSLiM_SLiMgui.h"
+#include "QtSLiM_Plot.h"
 #include "QtSLiMScriptTextEdit.h"
 #include "QtSLiMAbout.h"
 #include "QtSLiMPreferences.h"
@@ -62,6 +63,7 @@
 #include "eidos_globals.h"
 #include "eidos_beep.h"
 #include "slim_globals.h"
+#include "slim_globals.h"
 
 
 // Check the Qt version and display an error if it is unacceptable.  This can be turned off with
@@ -172,6 +174,9 @@ QtSLiMAppDelegate::QtSLiMAppDelegate(QObject *p_parent) : QObject(p_parent)
     Eidos_WarmUp();
     SLiM_WarmUp();
     
+    gSLiM_Plot_Class = new Plot_Class(gStr_Plot, gEidosDictionaryUnretained_Class);
+    gSLiM_Plot_Class->CacheDispatchTables();
+    
     gSLiM_SLiMgui_Class = new SLiMgui_Class(gStr_SLiMgui, gEidosDictionaryUnretained_Class);
     gSLiM_SLiMgui_Class->CacheDispatchTables();
 
@@ -837,6 +842,18 @@ void QtSLiMAppDelegate::addActionsForGlobalMenuItems(QWidget *window)
         connect(actionShowCycle_nonWF, &QAction::triggered, qtSLiMAppDelegate, &QtSLiMAppDelegate::dispatch_showCycle_nonWF);
         window->addAction(actionShowCycle_nonWF);
     }
+    {
+        QAction *actionShowCycle_WF_MS = new QAction("Show WF Tick Cycle (Multispecies)", this);
+        //actionAbout->setShortcut(Qt::CTRL + Qt::Key_Comma);
+        connect(actionShowCycle_WF_MS, &QAction::triggered, qtSLiMAppDelegate, &QtSLiMAppDelegate::dispatch_showCycle_WF_MS);
+        window->addAction(actionShowCycle_WF_MS);
+    }
+    {
+        QAction *actionShowCycle_nonWF_MS = new QAction("Show nonWF Tick Cycle (Multispecies)", this);
+        //actionAbout->setShortcut(Qt::CTRL + Qt::Key_Comma);
+        connect(actionShowCycle_nonWF_MS, &QAction::triggered, qtSLiMAppDelegate, &QtSLiMAppDelegate::dispatch_showCycle_nonWF_MS);
+        window->addAction(actionShowCycle_nonWF_MS);
+    }
     {
         QAction *actionHelp = new QAction("Help", this);
         //actionHelp->setShortcut(Qt::CTRL + Qt::Key_Comma);
@@ -1232,6 +1249,30 @@ void QtSLiMAppDelegate::dispatch_showCycle_nonWF(void)
     }
 }
 
+void QtSLiMAppDelegate::dispatch_showCycle_WF_MS(void)
+{
+    QWidget *imageWindow = globalImageWindowWithPath(":/help/TickCycle_WF_MS.png", "WF Cycle (Multispecies)", 0.32);
+    
+    if (imageWindow)
+    {
+        imageWindow->show();
+        imageWindow->raise();
+        imageWindow->activateWindow();
+    }
+}
+
+void QtSLiMAppDelegate::dispatch_showCycle_nonWF_MS(void)
+{
+    QWidget *imageWindow = globalImageWindowWithPath(":/help/TickCycle_nonWF_MS.png", "nonWF Cycle (Multispecies)", 0.32);
+    
+    if (imageWindow)
+    {
+        imageWindow->show();
+        imageWindow->raise();
+        imageWindow->activateWindow();
+    }
+}
+
 void QtSLiMAppDelegate::dispatch_help(void)
 {
     QtSLiMHelpWindow &helpWindow = QtSLiMHelpWindow::instance();
