@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 4/6/15; split from eidos_functions.cpp 09/26/2022
-//  Copyright (c) 2015-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2015-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -47,7 +47,7 @@ EidosValue_SP Eidos_ExecuteFunction_createDirectory(const std::vector<EidosValue
 	// Note that this function ignores matrix/array attributes, and always returns a vector, by design
 	
 	EidosValue *path_value = p_arguments[0].get();
-	std::string base_path = path_value->StringAtIndex(0, nullptr);
+	std::string base_path = path_value->StringAtIndex_NOCAST(0, nullptr);
 	std::string error_string;
 	bool success = Eidos_CreateDirectory(base_path, &error_string);
 	
@@ -67,7 +67,7 @@ EidosValue_SP Eidos_ExecuteFunction_deleteFile(const std::vector<EidosValue_SP>
 	EidosValue_SP result_SP(nullptr);
 	
 	EidosValue *filePath_value = p_arguments[0].get();
-	std::string base_path = filePath_value->StringAtIndex(0, nullptr);
+	std::string base_path = filePath_value->StringAtIndex_NOCAST(0, nullptr);
 	std::string file_path = Eidos_ResolvedPath(base_path);
 	
 	result_SP = ((remove(file_path.c_str()) == 0) ? gStaticEidosValue_LogicalT : gStaticEidosValue_LogicalF);
@@ -83,7 +83,7 @@ EidosValue_SP Eidos_ExecuteFunction_fileExists(const std::vector<EidosValue_SP>
 	EidosValue_SP result_SP(nullptr);
 	
 	EidosValue *filePath_value = p_arguments[0].get();
-	std::string base_path = filePath_value->StringAtIndex(0, nullptr);
+	std::string base_path = filePath_value->StringAtIndex_NOCAST(0, nullptr);
 	std::string file_path = Eidos_ResolvedPath(base_path);
 	
 	struct stat file_info;
@@ -102,11 +102,11 @@ EidosValue_SP Eidos_ExecuteFunction_filesAtPath(const std::vector<EidosValue_SP>
 	EidosValue_SP result_SP(nullptr);
 	
 	EidosValue *path_value = p_arguments[0].get();
-	std::string base_path = path_value->StringAtIndex(0, nullptr);
+	std::string base_path = path_value->StringAtIndex_NOCAST(0, nullptr);
 	int base_path_length = (int)base_path.length();
 	bool base_path_ends_in_slash = (base_path_length > 0) && (base_path[base_path_length-1] == '/');
 	std::string path = Eidos_ResolvedPath(base_path);
-	bool fullPaths = p_arguments[1]->LogicalAtIndex(0, nullptr);
+	bool fullPaths = p_arguments[1]->LogicalAtIndex_NOCAST(0, nullptr);
 	
 	// this code modified from GNU: http://www.gnu.org/software/libc/manual/html_node/Simple-Directory-Lister.html#Simple-Directory-Lister
 	// I'm not sure if it works on Windows... sigh...
@@ -116,7 +116,7 @@ EidosValue_SP Eidos_ExecuteFunction_filesAtPath(const std::vector<EidosValue_SP>
 	
 	if (dp != NULL)
 	{
-		EidosValue_String_vector *string_result = new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector();
+		EidosValue_String *string_result = new (gEidosValuePool->AllocateChunk()) EidosValue_String();
 		result_SP = EidosValue_SP(string_result);
 		
 		while (true)
@@ -166,7 +166,7 @@ EidosValue_SP Eidos_ExecuteFunction_getwd(__attribute__((unused)) const std::vec
 	EidosValue_SP result_SP(nullptr);
 	std::string cwd = Eidos_CurrentDirectory();
 	
-	result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(cwd));
+	result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(cwd));
 	
 	return result_SP;
 }
@@ -179,7 +179,7 @@ EidosValue_SP Eidos_ExecuteFunction_readFile(const std::vector<EidosValue_SP> &p
 	EidosValue_SP result_SP(nullptr);
 	
 	EidosValue *filePath_value = p_arguments[0].get();
-	std::string base_path = filePath_value->StringAtIndex(0, nullptr);
+	std::string base_path = filePath_value->StringAtIndex_NOCAST(0, nullptr);
 	std::string file_path = Eidos_ResolvedPath(base_path);
 	
 	// read the contents in
@@ -193,7 +193,7 @@ EidosValue_SP Eidos_ExecuteFunction_readFile(const std::vector<EidosValue_SP> &p
 	}
 	else
 	{
-		EidosValue_String_vector *string_result = new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector();
+		EidosValue_String *string_result = new (gEidosValuePool->AllocateChunk()) EidosValue_String();
 		result_SP = EidosValue_SP(string_result);
 		
 		std::string line;
@@ -220,12 +220,12 @@ EidosValue_SP Eidos_ExecuteFunction_setwd(const std::vector<EidosValue_SP> &p_ar
 	// Get the path; this code is identical to getwd() above, except it makes the value invisible
 	std::string cwd = Eidos_CurrentDirectory();
 	
-	result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(cwd));
+	result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(cwd));
 	result_SP->SetInvisible(true);
 	
 	// Now set the path
 	EidosValue *filePath_value = p_arguments[0].get();
-	std::string base_path = filePath_value->StringAtIndex(0, nullptr);
+	std::string base_path = filePath_value->StringAtIndex_NOCAST(0, nullptr);
 	std::string final_path = Eidos_ResolvedPath(base_path);
 	
 	errno = 0;
@@ -240,7 +240,7 @@ EidosValue_SP Eidos_ExecuteFunction_setwd(const std::vector<EidosValue_SP> &p_ar
 //	(string$)tempdir(void)
 EidosValue_SP Eidos_ExecuteFunction_tempdir(__attribute__((unused)) const std::vector<EidosValue_SP> &p_arguments, __attribute__((unused)) EidosInterpreter &p_interpreter)
 {
-	return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(Eidos_TemporaryDirectory()));
+	return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(Eidos_TemporaryDirectory()));
 }
 
 //	(logical$)flushFile(string$ filePath)
@@ -249,8 +249,11 @@ EidosValue_SP Eidos_ExecuteFunction_flushFile(const std::vector<EidosValue_SP> &
 	// Note that this function ignores matrix/array attributes, and always returns a vector, by design
 	
 	EidosValue *filePath_value = p_arguments[0].get();
-	std::string base_path = filePath_value->StringAtIndex(0, nullptr);
-	std::string file_path = Eidos_ResolvedPath(base_path);
+	std::string base_path = filePath_value->StringAtIndex_NOCAST(0, nullptr);
+	
+	// resolve the user-supplied path to a canonical absolute path, so that it remains valid even if the current directory changes
+	// this is needed because we buffer writes in some cases, and don't actually write them out until a later time
+	std::string file_path = Eidos_AbsolutePath(base_path);
 	
 	// note that writeFile() adds ".gz" to the filename if compression is specified and it is not already present; we don't,
 	// since we don't know if compression is on for this file; the user will therefore have to use the correct path
@@ -267,8 +270,11 @@ EidosValue_SP Eidos_ExecuteFunction_writeFile(const std::vector<EidosValue_SP> &
 	// Note that this function ignores matrix/array attributes, and always returns a vector, by design
 	
 	EidosValue *filePath_value = p_arguments[0].get();
-	std::string base_path = filePath_value->StringAtIndex(0, nullptr);
-	std::string file_path = Eidos_ResolvedPath(base_path);
+	std::string base_path = filePath_value->StringAtIndex_NOCAST(0, nullptr);
+	
+	// resolve the user-supplied path to a canonical absolute path, so that it remains valid even if the current directory changes
+	// this is needed because we buffer writes in some cases, and don't actually write them out until a later time
+	std::string file_path = Eidos_AbsolutePath(base_path);
 	
 	// the second argument is the file contents to write, which we put into a vector
 	EidosValue_String *contents_value = (EidosValue_String *)p_arguments[1].get();
@@ -278,13 +284,13 @@ EidosValue_SP Eidos_ExecuteFunction_writeFile(const std::vector<EidosValue_SP> &
 	contents_buffer.reserve(contents_count);
 	
 	for (int value_index = 0; value_index < contents_count; ++value_index)
-		contents_buffer.emplace_back(&contents_value->StringRefAtIndex(value_index, nullptr));
+		contents_buffer.emplace_back(&contents_value->StringRefAtIndex_NOCAST(value_index, nullptr));
 	
 	// the third argument is an optional append flag, F by default
-	bool append = p_arguments[2]->LogicalAtIndex(0, nullptr);
+	bool append = p_arguments[2]->LogicalAtIndex_NOCAST(0, nullptr);
 	
 	// and then there is a flag for optional gzip compression
-	bool do_compress = p_arguments[3]->LogicalAtIndex(0, nullptr);
+	bool do_compress = p_arguments[3]->LogicalAtIndex_NOCAST(0, nullptr);
 	
 	if (do_compress && !Eidos_string_hasSuffix(file_path, ".gz"))
 		file_path.append(".gz");
@@ -302,7 +308,7 @@ EidosValue_SP Eidos_ExecuteFunction_writeFile(const std::vector<EidosValue_SP> &
 		slimgui_buffer.reserve(contents_count);
 		
 		for (int value_index = 0; value_index < contents_count; ++value_index)
-			slimgui_buffer.emplace_back(contents_value->StringRefAtIndex(value_index, nullptr));
+			slimgui_buffer.emplace_back(contents_value->StringRefAtIndex_NOCAST(value_index, nullptr));
 		
 		if (context)
 			context->FileWriteNotification(file_path, std::move(slimgui_buffer), append);
@@ -325,16 +331,16 @@ EidosValue_SP Eidos_ExecuteFunction_writeTempFile(const std::vector<EidosValue_S
 		EIDOS_TERMINATION << "ERROR (Eidos_ExecuteFunction_writeTempFile): in function writeTempFile(), the temporary directory appears not to exist or is not writeable." << EidosTerminate(nullptr);
 	
 	EidosValue *prefix_value = p_arguments[0].get();
-	std::string prefix = prefix_value->StringAtIndex(0, nullptr);
+	std::string prefix = prefix_value->StringAtIndex_NOCAST(0, nullptr);
 	EidosValue *suffix_value = p_arguments[1].get();
-	std::string suffix = suffix_value->StringAtIndex(0, nullptr);
+	std::string suffix = suffix_value->StringAtIndex_NOCAST(0, nullptr);
 	
 	// the third argument is the file contents to write
 	EidosValue_String *contents_value = (EidosValue_String *)p_arguments[2].get();
 	int contents_count = contents_value->Count();
 	
 	// and then there is a flag for optional gzip compression
-	bool do_compress = p_arguments[3]->LogicalAtIndex(0, nullptr);
+	bool do_compress = p_arguments[3]->LogicalAtIndex_NOCAST(0, nullptr);
 	
 	if (do_compress && !Eidos_string_hasSuffix(suffix, ".gz"))
 		suffix.append(".gz");
@@ -367,7 +373,7 @@ EidosValue_SP Eidos_ExecuteFunction_writeTempFile(const std::vector<EidosValue_S
 		slimgui_buffer.reserve(contents_count);
 		
 		for (int value_index = 0; value_index < contents_count; ++value_index)
-			slimgui_buffer.emplace_back(contents_value->StringRefAtIndex(value_index, nullptr));
+			slimgui_buffer.emplace_back(contents_value->StringRefAtIndex_NOCAST(value_index, nullptr));
 		
 		if (context)
 			context->FileWriteNotification(file_path, std::move(slimgui_buffer), false);
@@ -393,11 +399,12 @@ EidosValue_SP Eidos_ExecuteFunction_writeTempFile(const std::vector<EidosValue_S
 			
 			if (contents_count == 1)
 			{
-				outstream << contents_value->StringAtIndex(0, nullptr);
+				// no final newline in this case, so the user can precisely specify the file contents if desired
+				outstream << contents_value->StringAtIndex_NOCAST(0, nullptr);
 			}
 			else
 			{
-				const std::vector<std::string> &string_vec = *contents_value->StringVector();
+				const std::string *string_vec = contents_value->StringData();
 				
 				for (int value_index = 0; value_index < contents_count; ++value_index)
 					outstream << string_vec[value_index] << std::endl;
@@ -433,7 +440,7 @@ EidosValue_SP Eidos_ExecuteFunction_writeTempFile(const std::vector<EidosValue_S
 			else
 			{
 				std::string file_path(file_path_cstr);
-				result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(file_path));
+				result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(file_path));
 			}
 		}
 	}
@@ -454,11 +461,12 @@ EidosValue_SP Eidos_ExecuteFunction_writeTempFile(const std::vector<EidosValue_S
 		{
 			if (contents_count == 1)
 			{
-				file_stream << contents_value->StringAtIndex(0, nullptr);	// no final newline in this case, so the user can precisely specify the file contents if desired
+				// no final newline in this case, so the user can precisely specify the file contents if desired
+				file_stream << contents_value->StringAtIndex_NOCAST(0, nullptr);
 			}
 			else
 			{
-				const std::vector<std::string> &string_vec = *contents_value->StringVector();
+				const std::string *string_vec = contents_value->StringData();
 				
 				for (int value_index = 0; value_index < contents_count; ++value_index)
 				{
@@ -479,7 +487,7 @@ EidosValue_SP Eidos_ExecuteFunction_writeTempFile(const std::vector<EidosValue_S
 			}
 			else
 			{
-				result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(file_path));
+				result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(file_path));
 			}
 		}
 	}
